use std::time::Duration;

use bevy::{input::mouse::MouseWheel, math::NormedVectorSpace, picking::{focus::HoverMap, pointer::PointerId}, prelude::*, utils::HashMap, window::PrimaryWindow};

use crate::ui;

pub struct Plugin;
impl bevy::prelude::Plugin for Plugin {
    fn build(&self, app: &mut App) {
        app.add_systems(Startup, setup);
            // .add_systems(Update, move_cam)
            // .init_resource::<Pointers>();
    }
}

fn setup(mut cmds: Commands, window: Query<&Window>) {
    let zoom = 2./window.single().width().min(window.single().height());
    cmds.spawn((
        Camera2d,
        Transform::from_scale(Vec3::new(zoom, zoom, zoom))
    ));
}

#[derive(Resource, Default)]
struct Pointers(HashMap<PointerId, (Vec2, Option<Duration>)>);

fn move_cam(
    mut cam: Query<&mut Transform, With<Camera2d>>,
    map_ui_entity: Query<Entity, With<ui::MapUIComponent>>,
    mouse_buttons: Res<ButtonInput<MouseButton>>,
    mut ev_scroll: EventReader<MouseWheel>,
    touches: Res<Touches>,
    window: Query<&Window, With<PrimaryWindow>>,
    mut pointers: ResMut<Pointers>,
    hover_map: Res<HoverMap>,
    time: Res<Time>
) {
    let window = window.single();
    let mut cam = cam.single_mut();
    let map_ui_entity = map_ui_entity.single();
    let ps = hover_map.iter().filter_map(|(id, hit_map)| match id {
            PointerId::Mouse => window.cursor_position().map(|p| 
                match pointers.0.get(id) {
                    Some(p_cache) => (p_cache.1.filter(|_| mouse_buttons.pressed(MouseButton::Left)), p, p_cache.0),
                    None => (None, p, p)
                }
            ),
            PointerId::Touch(i) => touches.get_pressed(*i).map(|t| (pointers.0.get(id).map(|(pos, start)| (*start).unwrap_or(time.elapsed())), t.position(), t.previous_position())),
            _ => None
        }.map(|(pressed_start,new_pos, old_pos)| (pressed_start,new_pos,old_pos,id,hit_map))
    ).collect::<Vec<_>>();
    let pressed_on_map = ps.iter().filter(|p| p.0.is_some() && p.4.contains_key(&map_ui_entity)).collect::<Vec<_>>();
    
    let old_midpoint = pressed_on_map.iter().fold(Vec2::ZERO, |acc, (_, _, old_pos, _, _)| {
        acc + (old_pos/pressed_on_map.len() as f32)
    });
    let new_midpoint = pressed_on_map.iter().fold(Vec2::ZERO, |acc, (_, new_pos, _, _, _)| {
        acc + (new_pos/pressed_on_map.len() as f32)
    });

    // move camera
    cam.translation.x -= (new_midpoint.x - old_midpoint.x)*cam.scale.x;
    cam.translation.y += (new_midpoint.y - old_midpoint.y)*cam.scale.y;
    
    // multiple fingers zoom
    if pressed_on_map.len() > 1 {
        let old_d_to_midpoint = pressed_on_map.iter().fold(0., |acc, (_, _, old_pos, _, _)| {
            acc + (old_midpoint-old_pos).norm()
        });
        let new_d_to_midpoint = pressed_on_map.iter().fold(0., |acc, (_, new_pos, _, _, _)| {
            acc + (new_midpoint-new_pos).norm()
        });
        let zoom = new_d_to_midpoint/old_d_to_midpoint;
        cam.scale /= zoom;
    }

    // mouse scroll zoom
    for ev in ev_scroll.read() {
        let scale = (cam.scale.x-(ev.y*0.1/window.width().min(window.height()))).clamp(0.0001, 2./window.width().min(window.height()));
        cam.scale = Vec3::new(scale, scale, scale);
    }

    // update cached pointer positions
    pointers.0.clear();
    for (pressed_start, new_pos, _, id, _) in ps {
        match id {
            PointerId::Mouse => {pointers.0.insert(*id, (new_pos, pressed_start));},
            PointerId::Touch(_) => {pointers.0.insert(*id, (new_pos, pressed_start));},
            _ => {}
        }
    }
}